@@ -6,9 +6,7 @@ print STDERR "Processing sample 1-1\n";
 
 @data = ( 
     ["1st","2nd","3rd","4th","5th","6th","7th", "8th", "9th"],
-    #[    1,    2,    5,    6,    3,  1.5,    1,     3,     4],
-    [    0,    0,    0,    0,    0,    0,    0,     0,     0],
-    [    0,    0,    0,    0,    0,    0,    0,     0,     0],
+    [    1,    2,    5,    6,    3,  1.5,    1,     3,     4],
 );
 
 $my_graph = new GD::Graph::bars();
@@ -20,7 +18,13 @@ $my_graph->set(
 	y_max_value => 8,
 	y_tick_number => 8,
 	y_label_skip => 2,
-);
+	
+	# shadows
+	bar_spacing => 8,
+	shadow_depth => 4,
+	shadowclr => 'dred',
+) 
+or warn $my_graph->error;
 
 $my_graph->plot(\@data);
 save_chart($my_graph, 'sample11');
